@@ -24,8 +24,6 @@
 
 
 
-/*local vars */
-static int always_echo = FALSE;
 
 /* global vars */
 int slave_pty_sensing_fd = -1;       /* slave pty: client uses this to communicate with rlwrap,
@@ -41,27 +39,32 @@ my_pty_fork(int *ptr_master_fd,
   int fdm, fds = -1;
   int ttyfd, timeout;
   pid_t pid;
-  const char *slave_name;
+  const void *slave_name_or_ptytty; /* opaque PTYTTY pointer when using libptytty, else slave name */
   struct termios pterm;
   int only_sigchld[] = { SIGCHLD, 0 };
   
 
 
-  ptytty_openpty(&fdm, &fds, &slave_name);
+  ptytty_openpty(&fdm, &fds, &slave_name_or_ptytty);
 
 
   block_signals(only_sigchld);  /* block SIGCHLD until we have had a chance to install a handler for it after the fork() */
-
+  fflush(NULL);
   if ((pid = fork()) < 0) {
     myerror(FATAL|USE_ERRNO, "Cannot fork");
     return(42); /* the compiler may not know that myerror() won't return */
   } else if (pid == 0) {                /* child */
     DEBUG_RANDOM_SLEEP;
     i_am_child = TRUE;          /* remember who I am */
-    unblock_all_signals();
-    
+    if (debug)
+        my_fopen(&debug_fp, DEBUG_FILENAME, "a+", "debug log");
+
+    unblock_all_signals();    
     close(fdm);                 /* fdm not used in child */
-    ptytty_control_tty(fds, slave_name);
+    if (skip_setctty) 
+      DPRINTF0(DEBUG_TERMIO, "--skip-setccty: don't make slave pty a controlling terminal");
+    else
+      ptytty_control_tty(fds, slave_name_or_ptytty);
 
     if (dup2(fds, STDIN_FILENO) != STDIN_FILENO) /* extremely unlikely */
       myerror(FATAL|USE_ERRNO, "dup2 to stdin failed");
@@ -82,10 +85,8 @@ my_pty_fork(int *ptr_master_fd,
   } else {                      /* parent */
     srand(pid); DEBUG_RANDOM_SLEEP;
     command_pid = pid;            /* the SIGCHLD signal handler needs this global variable */
- 
     *ptr_master_fd = fdm;
 
-    
     /* Try to find a suitable file descriptor to sense the state of the slave pty                                              */
     /* At least on Solaris, we have to use the slave pty itself, but this works only after it has been set up in the child     */
     /* This may take some time, hence the (increasing) timeout below                                                           */
@@ -125,11 +126,14 @@ my_pty_fork(int *ptr_master_fd,
 
     errno = 0;
     if (renice) {
+#if HAVE_DECL_NICE
       int retval = nice(1);
       if (retval < 0 && errno)  
         myerror(FATAL|USE_ERRNO, "could not increase my own niceness"); 
+#else
+      myerror(FATAL, "This system doesnt't have the nice (2) function");
+#endif
     }
-
     if (slave_winsize != NULL)
       if (ioctl(slave_pty_sensing_fd, TIOCSWINSZ, slave_winsize) < 0) 
         myerror(FATAL|USE_ERRNO, "TIOCSWINSZ failed on %s pty", sensing_pty); /* This is done in parent and not in child as that would fail on Solaris (why?) */ 
@@ -140,14 +144,14 @@ my_pty_fork(int *ptr_master_fd,
 
 
 int
-slave_is_in_raw_mode()
+slave_is_in_raw_mode(void)
 {
   struct termios *pterm_slave;
   static int been_warned = 0;
   int in_raw_mode;
-  if (always_echo)
-    return FALSE;
 
+  
+  
   if (command_is_dead)
     return FALSE; /* filter last words  too (even if ncurses-ridden) */
   if (!(pterm_slave = my_tcgetattr(slave_pty_sensing_fd, "slave pty"))) {
@@ -164,14 +168,13 @@ slave_is_in_raw_mode()
 }
 
 void
-mirror_slaves_echo_mode()
+mirror_slaves_echo_mode(void)
 {                               /* important e.g. when slave command asks for password  */
   struct termios *pterm_slave = NULL;
   int should_echo_anyway = always_echo || (always_readline && !dont_wrap_command_waits());
 
   if ( !(pterm_slave = my_tcgetattr(slave_pty_sensing_fd, "slave pty")) ||
-       command_is_dead ||
-       always_echo 
+       command_is_dead 
        )
     /* race condition here: SIGCHLD may not yet have been caught */
     return;
@@ -187,7 +190,6 @@ mirror_slaves_echo_mode()
   if (should_echo_anyway && password_prompt_search_string) {
     char *p, *q;
     DPRINTF2(DEBUG_READLINE, "matching prompt <%s> and password search string <%s>..", saved_rl_state.raw_prompt, password_prompt_search_string); 
-    assert(strlen(saved_rl_state.raw_prompt) < BUFFSIZE);
     p = saved_rl_state.raw_prompt + strlen(saved_rl_state.raw_prompt) - 1;
     q =
       password_prompt_search_string + strlen(password_prompt_search_string) -
@@ -218,7 +220,7 @@ mirror_slaves_echo_mode()
 }
 
 void
-write_EOF_to_master_pty()
+write_EOF_to_master_pty(void)
 {
   struct termios *pterm_slave = my_tcgetattr(slave_pty_sensing_fd, "slave pty");
   char *sent_EOF = mysavestring("?");
@@ -258,7 +260,7 @@ write_EOL_to_master_pty(char *received_eol)
 }
 
 void
-completely_mirror_slaves_terminal_settings()
+completely_mirror_slaves_terminal_settings(void)
 {
   
   struct termios *pterm_slave;
@@ -272,7 +274,7 @@ completely_mirror_slaves_terminal_settings()
 }
 
 void
-completely_mirror_slaves_output_settings() 
+completely_mirror_slaves_output_settings(void) 
 {
   struct termios *pterm_stdin, *pterm_slave;  
   DEBUG_RANDOM_SLEEP;
@@ -313,7 +315,7 @@ completely_mirror_slaves_output_settings()
 */  
 
 void
-completely_mirror_slaves_special_characters()
+completely_mirror_slaves_special_characters(void)
 {
   struct termios *pterm_stdin, *pterm_slave;
   DEBUG_RANDOM_SLEEP;
@@ -338,19 +340,24 @@ completely_mirror_slaves_special_characters()
 }
 
 
-/* returns TRUE if the -N option has been specified, we can read /proc/<command_pid>/wchan,
-   (which happens only on linux, as far as I know) and what we read there contains the word "wait"
-   meaning (presumably...) that command is waiting for one of its children
-   if otherwise returns FALSE
+/* returns TRUE if the -N option has been specified, we can read
+   /proc/<command_pid>/wchan, (which happens only on linux, as far as
+   I know) and what we read there contains one of the "wait_prhases"m
+   meaning (presumably...) that command is waiting for one of its
+   children if otherwise returns FALSE
 */
-int dont_wrap_command_waits() {
+
+  
+int dont_wrap_command_waits(void) {
   static char command_wchan[MAXPATHLEN+1];
+  static char *wait_phrases[] = {"wait4", "suspend", "do_wait", NULL}; 
   static int initialised = FALSE;
   static int wchan_fd;
   static int been_warned = 0;
-  char buffer[BUFFSIZE];
+  char buffer[BUFFSIZE], **p;
   int nread, result = FALSE;
 
+  
   DEBUG_RANDOM_SLEEP;
   if (!commands_children_not_wrapped)
     return FALSE;
@@ -360,21 +367,32 @@ int dont_wrap_command_waits() {
   }
   if (command_is_dead)
     return TRUE;  /* This is lazy!! signal may not have been delivered @@@ */
+
+  if (screen_is_alternate)
+    return TRUE;  /* assume that programs that use the alternate screen never need rlwrap */
+
+  
   wchan_fd =  open(command_wchan, O_RDONLY);
-  if (wchan_fd < 0) { 
-    if (been_warned++ == 0) {
-      myerror(WARNING|USE_ERRNO, "you probably specified the -N (--no-children) option"
-                                 " - but spying\non %s's wait status does not work on"
-                                 " your system, as we cannot read %s", command_name, command_wchan);
+  if (wchan_fd < 0) { /* Even if screen is not alternate, the wrapped command might ask for a single keypress like "Continue Y/N?" */
+    if (been_warned++ == 0 && !(term_rmcup && term_smcup)) {
+      myerror(WARNING|NOERRNO, "you specified the -N (--no-children) option  - but spying\n"
+                               "on %s's wait status doesn't work on your system, as rlwrap\n "
+                               "cannot sense the 'alternate screen',and cannot read %s", command_name, command_wchan);
     }
     return FALSE;
-  }     
+  }
 
 
   if (((nread = read(wchan_fd, buffer, BUFFSIZE -1)) > 0)) {
     buffer[nread] =  '\0';
-    DPRINTF1(DEBUG_READLINE, "read commands wchan: <%s> ", buffer);
-    result = (strstr(buffer, "wait") != NULL);
+    result = FALSE;
+    for (p = wait_phrases; *p; p++) /* Not quite watertight: I don't have a complete (and unchanging) list
+                                       of syscalls that make client wait for one of its children */
+      if (strstr(buffer, *p)) {
+        result = TRUE;
+        break;
+      }
+    DPRINTF3(DEBUG_READLINE, "read commands wchan %s: <%s>, waiting: %s", command_wchan, buffer, result ? "yes" : "no");
   }
   close(wchan_fd);
   DEBUG_RANDOM_SLEEP;
@@ -382,12 +400,12 @@ int dont_wrap_command_waits() {
 }       
 
 
-int skip_rlwrap() { /* this function is called from sigTSTP signal handler. Is it re-entrant? */
+int skip_rlwrap(void) { /* this function is called from sigTSTP signal handler. Is it re-entrant? */
   int retval = FALSE;
   DEBUG_RANDOM_SLEEP;
   if (dont_wrap_command_waits())
     retval = TRUE;
-  else if (always_readline)
+  else  if (always_readline)
     retval =  FALSE;
   else if (slave_is_in_raw_mode())
     retval= TRUE;
