@@ -1,5 +1,5 @@
 /*  readline.c: interacting with the GNU readline library 
-    (C) 2000-2007 Hans Lub
+    (C) 2000-2025 Hans Lub
     
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -50,8 +50,6 @@ static int direct_keypress(int, int);
 static int direct_prefix(int, int);
 static int handle_hotkey(int, int);
 static int handle_hotkey_without_history(int, int);
-static int please_update_alaf(int,int);
-static int please_update_ce(int,int);
 
 /* only useful while debugging: */
 static int debug_ad_hoc(int,int);
@@ -78,11 +76,6 @@ init_readline(char *UNUSED(prompt))
   /* if someone's .inputrc binds a key to accept-line, make it use our own version in lieu of readline's */
   rl_add_defun("accept-line", my_accept_line, -1);
   
-
-  
-  /* the old rlwrap bindable function names with underscores are deprecated: */
-  rl_add_defun("rlwrap_accept_line_and_forget", please_update_alaf,-1);
-  rl_add_defun("rlwrap_call_editor", please_update_ce,-1);
   
   /* put the next variable binding(s) *before* rl_initialize(), so they can be overridden */
   rl_variable_bind("blink-matching-paren","on");  
@@ -103,8 +96,11 @@ init_readline(char *UNUSED(prompt))
 				   in .inputrc */
 
   /* put the next variable binding(s) *after* rl_initialize(), so they cannot be overridden */
-  /* rl_variable_bind("enable-bracketed-paste","off"); */     /* enable-bracketed-paste changes cursor positioning after printing the prompt ...
-                                                           ... causing rlwrap to overwrite it after accepting input                        */
+  if (rl_readline_version < 0x802)  /* in bracketed_paste_mode rl_deprep_terminal() prints BRACK_PASTE_FINI ("\033[?2004l\r") 
+                                       which moves the cursor to the beginning of the line. In pre-8.2 readline,  this doesn't
+                                       set _rl_last_c_pos to 0, which makes rl_redisplay() think that it is already past the prompt,
+                                       making it overwrite the prompt that rlwrap has already printed. cf. https://github.com/hanslub42/rlwrap/issues/168                    */
+      rl_variable_bind("enable-bracketed-paste","off");      
  
   using_history();
   rl_redisplay_function = my_redisplay;
@@ -123,6 +119,7 @@ init_readline(char *UNUSED(prompt))
   saved_rl_state.cooked_prompt = NULL;
 
   bracketed_paste_enabled = term_enable_bracketed_paste != NULL && strings_are_equal(rl_variable_value("enable-bracketed-paste"),"on");
+                           
   DPRINTF2(DEBUG_READLINE, "var: %s, enabled: %s", rl_variable_value("enable-bracketed-paste"), term_enable_bracketed_paste);
   if (bracketed_paste_enabled) {
     DPRINTF0(DEBUG_READLINE, "bracketed-paste is enabled");
@@ -192,7 +189,7 @@ message_in_echo_area(char *message)
   if (message) {
     rl_save_prompt();
     message_in_echo_area = TRUE;  
-    rl_message(message);
+    rl_message("%s", message);
   }  else {
     if (message_in_echo_area)
       rl_restore_prompt();
@@ -239,18 +236,10 @@ line_handler(char *line)
        O.K, we know for sure that cursor is at start of line (after the prompt, or at position 0, if bracketed paste is
        enabled)). When clients output arrives, it will be printed at just the right place - but first we 'll erase the
        user input (as it may be about to be changed by the filter) */
-
-    if (bracketed_paste_enabled) {   /* we're at start of line and the prompt has just been erased */
-      SHOWCURSOR('1');
-      move_cursor_to_start_of_prompt(FALSE);
-      SHOWCURSOR('2');
-      my_putstr(saved_rl_state.cooked_prompt);
-      SHOWCURSOR('3');
-    }
       
     rl_delete_text(0, rl_end);  /* clear line  (after prompt) */
     rl_point = 0;
-    my_redisplay();             /* and redisplay (this time without user input, cf the comments for the line_handler() function below) */
+    my_redisplay();             /* and redisplay (now without user input, which will be echoed back, cf. comments above) */
     rewritten_line =
       (multiline_separator ? 
        search_and_replace(multiline_separator, "\n", line, 0, NULL,
@@ -310,8 +299,9 @@ line_handler(char *line)
 
     /* readline only outputs term_enable_bracketed_paste when we call rl_prep_terminal(). That's too   */
     /* late for us, as we only call rl_prep_terminal *after* we have received user input               */
+
     if (bracketed_paste_enabled)
-      my_putstr(term_enable_bracketed_paste);
+       my_putstr(term_enable_bracketed_paste); 
   }
 }
 
@@ -368,6 +358,7 @@ my_add_history(char *line)
       lookback = 1; break;
     case ELIMINATE_ALL_DOUBLES:
       lookback = history_length; break;
+    default: lookback = 0;
     }
 
   
@@ -426,7 +417,7 @@ my_homegrown_redisplay(int hide_passwords)
   int i, printed_length,
     new_curpos,                    /* cursor position on screen */
     keep_old_line, vlinestart, printwidth, last_column;
-  DPRINTF3(DEBUG_AD_HOC,"rl_prompt: <%s>, prompt_without_ignore_markers: <%s>,  prompt_plus_line: <%s>", rl_prompt, prompt_without_ignore_markers, prompt_plus_line);   
+  DPRINTF3(DEBUG_AD_HOC,"hgrown: rl_prompt: <%s>, prompt_without_ignore_markers: <%s>,  prompt_plus_line: <%s>", rl_prompt, prompt_without_ignore_markers, prompt_plus_line);   
 
   /* In order to handle prompt with colour we either print the whole prompt, or start past it:
      starting in the middle is too difficult (i.e. I am too lazy) to get it right.
@@ -551,7 +542,7 @@ my_redisplay(void)
 }
 
 
-
+/* (setq font-lock-maximum-decoration t) */
 
 static void
 munge_file_in_editor(const char *filename, int lineno, int colno)
@@ -871,12 +862,13 @@ colourise (const char *prompt)
   return result;
 }
 
+
 void
 move_cursor_to_start_of_prompt(int erase)
 {
   int termwidth = winsize.ws_col;
   int promptlen_on_screen, number_of_lines_in_prompt, curpos, count;
-  int cooked = (saved_rl_state.cooked_prompt != NULL);
+  int ATTR_ONLY_FOR_DEBUGGING cooked = (saved_rl_state.cooked_prompt != NULL);
   
   DPRINTF2(DEBUG_READLINE,"prompt_is_still_uncooked: %d, impatient_prompt: %d", prompt_is_still_uncooked, impatient_prompt);
   if (prompt_is_still_uncooked && ! impatient_prompt)
@@ -1035,24 +1027,6 @@ int cook_prompt_if_necessary (void) {
 
 
 
-/* Utility functions for binding keys. */
-
-static int please_update(const char *varname) {
-  myerror(FATAL|NOERRNO, "since version 0.35, the readline function '%s' is called '%s'\n"
-          "(hyphens instead of underscores). Please update your .inputrc",
-          varname, search_and_replace("_","-", varname, 0, NULL, NULL));
-  return 0;
-}       
-
-static int please_update_alaf(int UNUSED(count), int UNUSED(key)) {
-  return please_update("rlwrap_accept_line_and_forget");
-}
-
-static int please_update_ce(int UNUSED(count), int UNUSED(key)) {
-  return please_update("rlwrap_call_editor");
-}
-
-
 
 /*
    From the readline documentation: Acceptable keymap names are emacs, emacs-standard, emacs-meta, emacs-ctlx, vi,
