@@ -20,8 +20,16 @@
 */
 
 
+#if !defined(__STDC_VERSION__) || __STDC_VERSION__ < 199901L
+  typedef int bool;
+  #define true 1
+  #define false 0
+#endif
+
+
 #include "../config.h"
 #include <sys/types.h>
+
 #if HAVE_SYS_WAIT_H
 #  include <sys/wait.h>
 #endif
@@ -31,7 +39,6 @@
 #endif
 
 
-
 #include <sys/stat.h>
 #include <sys/ioctl.h>
 #include <sys/resource.h>
@@ -59,15 +66,10 @@
 
 
 
-#if TIME_WITH_SYS_TIME
-# include <sys/time.h>
-# include <time.h>
-#else
-# if HAVE_SYS_TIME_H
+#if HAVE_SYS_TIME_H
 #  include <sys/time.h>
-# else
+#else
 #  include <time.h>
-# endif
 #endif
 
 #if HAVE_SYS_FILE_H
@@ -124,15 +126,16 @@ char *strchr(), *strrchr();
 #endif
 
 
-#ifdef HAVE_PTY_H /* glibc (even if BSD) */
-#  include <pty.h>
-#elif HAVE_LIBUTIL_H /* BSD, non-glibc */
-#  include <libutil.h>
-#elif HAVE_UTIL_H /* BSD, other varriants */
-#  include <util.h>
+#if ! USE_LIBPTYTTY
+#  ifdef HAVE_PTY_H /* glibc (even if BSD) */
+#    include <pty.h>
+#  elif HAVE_LIBUTIL_H /* BSD, non-glibc */
+#    include <libutil.h>
+#  elif HAVE_UTIL_H /* BSD, other varriants */
+#    include <util.h>
+#  endif
 #endif
 
-
 #if HAVE_DECL_PROC_PIDVNODEPATHINFO
 #  include <libproc.h>
 #endif
@@ -156,7 +159,7 @@ char *strchr(), *strrchr();
 #endif
 
 
-#ifdef  HAVE_SNPRINTF           /* don't rely on the compiler understanding variadic macros */
+#if  HAVE_DECL_SNPRINTF           /* don't rely on the compiler understanding variadic macros */
 # define snprintf0(buf,bufsize,format)                           snprintf(buf,bufsize,format)
 # define snprintf1(buf,bufsize,format,arg1)                      snprintf(buf,bufsize,format,arg1)
 # define snprintf2(buf,bufsize,format,arg1,arg2)                 snprintf(buf,bufsize,format,arg1,arg2)
@@ -182,7 +185,6 @@ char *strchr(), *strrchr();
 #endif
 
 
-
 #include <readline/readline.h>
 #include <readline/history.h>
 
@@ -215,6 +217,10 @@ extern int _rl_horizontal_scroll_mode;
 
 
 
+#define DEF_UNIT_TEST(name) void name(int argc, char **argv, test_stage s)
+#define STAGE(stage) s == stage
+
+
 
 extern void test_main(void);
 
@@ -266,6 +272,7 @@ extern int remember_for_completion;
 extern int commands_children_not_wrapped; 
 extern int accepted_lines;
 extern char *filter_command;
+extern int skip_setctty;
 extern int polling;
 
 void cleanup_rlwrap_and_exit(int status);
@@ -315,10 +322,7 @@ extern int deferred_adapt_commands_window_size;
 extern int signal_handlers_were_installed;
 extern int received_sigALRM;
 
-#ifndef RETSIGTYPE
-#define RETSIGTYPE void /* systems where RETSIGTYPE = int have died out, apparently */
-#endif
-typedef RETSIGTYPE (*sighandler_type)(int);       
+typedef void (*sighandler_type)(int);       
 
 /* we'll install signal handlers with mysignal(SIGBLAH, HANDLER(handler)), to improve debug log readabilty */
 void mysignal(int sig, sighandler_type handler, const char *handler_name);
@@ -417,7 +421,7 @@ void remove_padding_and_terminate(char *buf, int length);
 void unbackspace(char* buf);
 char *mark_invisible(const char *buf);
 char *copy_and_unbackspace(const char *original);
-int colourless_strlen(const char *str, char **pcopy_without_ignore_markers, int termwidth);
+int colourless_strlen(const char *str, char **pcopy_without_ignore_markers, int termwidth, int stop_at, char **stopptr);
 int colourless_strlen_unmarked (const char *str, int termwidth);
 char *get_last_screenline(char *long_line, int termwidth);
 char *lowercase(const char *str);
@@ -445,14 +449,14 @@ int dont_wrap_command_waits(void);
 int skip_rlwrap(void);
 
 /* in ptytty.c: */
-int ptytty_get_pty(int *fd_tty, const char **ttydev);
-int ptytty_get_tty(const char *ttydev);
-int ptytty_control_tty(int fd_tty, const char *ttydev);
-int ptytty_openpty(int *amaster, int *aslave, const char **name);
+
+int ptytty_control_tty(int fd_tty, const void *ptytty_or_ttydev);
+int ptytty_openpty(int *amaster, int *aslave, const void **ptytty_or_ttydev);
+
 
 /* in completion.rb: */
 void init_completer(void);
-void feed_file_into_completion_list(const char *completions_file);
+void feed_file_into_completion_list(const char *completions_file, bool warn_if_unreadable);
 void feed_line_into_completion_list(const char *line);
 void add_word_to_completions(const char *word);
 void remove_word_from_completions(const char *word);
@@ -597,6 +601,10 @@ typedef enum  {TEST_AT_PROGRAM_START, TEST_AFTER_OPTION_PARSING, TEST_AFTER_SPAW
 
 #define MAYBE_UNUSED(x) (void) (x)
 
+#define ATTR_ONLY_FOR_DEBUGGING __attribute__((__unused__)) /* some variables get defined, but are only ever used in DEBUG statements. USE this macro to prevent
+                                                               -Wunused-variables complaining*/
+ 
+
 #ifdef  DEBUG
 
 
@@ -681,6 +689,7 @@ typedef enum  {TEST_AT_PROGRAM_START, TEST_AFTER_OPTION_PARSING, TEST_AFTER_SPAW
 #  define SHOWCURSOR
 #  define DEBUG_RANDOM_SLEEP  
 
+
 /* Don't #define NDEBUG!  There are assertions that cannot be skipped, as in assert(important_function_call()) */
 /* Todo (maybe) #define myassert(x) if(DEBUG){assert(x)} for the other (skippable) assertions                  */
 
