@@ -86,7 +86,7 @@ mysignal(int sig, sighandler_type handler, const char *handler_name) {
 
 
 void
-install_signal_handlers()
+install_signal_handlers(void)
 {
   int i;
   mysignal(SIGCHLD, HANDLER(child_died));
@@ -105,7 +105,7 @@ install_signal_handlers()
 }
 
 
-static void uninstall_signal_handlers() {
+static void uninstall_signal_handlers(void) {
   int i;
   for(i=1; i<MAX_SIG; i++)
     mysignal(i, SIG_DFL, NULL);
@@ -114,7 +114,7 @@ static void uninstall_signal_handlers() {
 
 
 void
-ignore_sigchld()
+ignore_sigchld(void)
 {
   mysignal(SIGCHLD, HANDLER(do_nothing));
 }
@@ -161,7 +161,7 @@ change_signalmask(int how, int *sigs)
 
 
 void
-unblock_all_signals()
+unblock_all_signals(void)
 {
   sigset_t mask;
   sigfillset(&mask);
@@ -169,7 +169,7 @@ unblock_all_signals()
 } 
 
 void
-block_all_signals()
+block_all_signals(void)
 {
   sigset_t mask;
   int i;
@@ -340,7 +340,8 @@ void wipe_textarea(struct winsize *old_winsize)
 {
   int point, lineheight, linelength, cursor_height, i, promptlength;
   if (!prompt_is_single_line()) {       /* Don't need to do anything in horizontal_scroll_mode  */
-    promptlength = colourless_strlen((saved_rl_state.cooked_prompt ? saved_rl_state.cooked_prompt:  saved_rl_state.raw_prompt), NULL, old_winsize -> ws_col); 
+    promptlength = colourless_strlen((saved_rl_state.cooked_prompt ? saved_rl_state.cooked_prompt:  saved_rl_state.raw_prompt),
+                                     NULL, old_winsize -> ws_col, 0, NULL); 
     linelength = (within_line_edit ? strlen(rl_line_buffer) : 0) + promptlength;
     point = (within_line_edit ? rl_point : 0) + promptlength;
     assert(old_winsize -> ws_col > 0);
@@ -419,6 +420,7 @@ handle_program_error_signal(int sig)
 
 
 static int coredump(int status) {
+  MAYBE_UNUSED(status);
 #ifdef WCOREDUMP
   return WCOREDUMP(status);
 #else
@@ -432,7 +434,7 @@ void suicide_by(int signal, int status) {
      it didn't. @@@ We could also try argv[0] = command_name just before dying ? */
   
   if (signals_program_error(signal)) {  
-    myerror(WARNING|NOERRNO, "%s crashed, killed by %s%s.\n%s itself has not crashed, but for transparency,\n"
+    myerror(WARNING|NOERRNO, "%s crashed, killed by %s%s.\n%s itself has not crashed but, for transparency,\n"
            "it will now kill itself %swith the same signal\n",
            command_name, signal_name(signal), (coredump(status) ? " (core dumped)" : ""),
            program_name, (coredump(status) ? "" : "(without dumping core) ") );
@@ -486,26 +488,26 @@ handle_sigALRM(int UNUSED(signo)) {
    architectures signal values may coincide */
 char *signal_name(int signal) {
   return
-    signal ==  SIGHUP  ? "SIGHUP" :
-    signal ==  SIGINT  ? "SIGINT" :
+    signal ==  SIGHUP   ? "SIGHUP" :
+    signal ==  SIGINT   ? "SIGINT" :
     signal ==  SIGQUIT  ? "SIGQUIT" :
-    signal ==  SIGILL  ? "SIGILL" :
+    signal ==  SIGILL   ? "SIGILL" :
     signal ==  SIGABRT  ? "SIGABRT" :
     signal ==  SIGTRAP  ? "SIGTRAP" :
 #ifdef SIGIOT  /* 4.2 BSD (IOT trap ) */
-    signal ==  SIGIOT  ? "SIGIOT" :
+    signal ==  SIGIOT   ? "SIGIOT" :
 #endif
 #ifdef SIGEMT  /* 4.2 BSD (EMT trap ) */
-    signal ==  SIGEMT  ? "SIGEMT" :
+    signal ==  SIGEMT   ? "SIGEMT" :
 #endif
-    signal ==  SIGFPE  ? "SIGFPE" :
+    signal ==  SIGFPE   ? "SIGFPE" :
     signal ==  SIGKILL  ? "SIGKILL" :
 #ifdef SIGBUS  /* 4.2 BSD (Bus error ) */
-    signal ==  SIGBUS  ? "SIGBUS" :
+    signal ==  SIGBUS   ? "SIGBUS" :
 #endif
     signal ==  SIGSEGV  ? "SIGSEGV" :
 #ifdef SIGSYS  /* 4.2 BSD (Bad argument to system call ) */
-    signal ==  SIGSYS  ? "SIGSYS" :
+    signal ==  SIGSYS   ? "SIGSYS" :
 #endif
     signal ==  SIGPIPE  ? "SIGPIPE" :
     signal ==  SIGALRM  ? "SIGALRM" :
@@ -517,10 +519,10 @@ char *signal_name(int signal) {
     signal ==  SIGTSTP  ? "SIGTSTP" :
     signal ==  SIGCONT  ? "SIGCONT" :
 #ifdef SIGCLD  /* System V (Same as SIGCHLD ) */
-    signal ==  SIGCLD  ? "SIGCLD" :
+    signal ==  SIGCLD   ? "SIGCLD" :
 #endif
 #ifdef SIGPWR  /* System V (Power failure restart ) */
-    signal ==  SIGPWR  ? "SIGPWR" :
+    signal ==  SIGPWR   ? "SIGPWR" :
 #endif
     signal ==  SIGXCPU  ? "SIGXCPU" :
     signal ==  SIGXFSZ  ? "SIGXFSZ" :
