@@ -1,5 +1,5 @@
 /* rlwrap - a readline wrapper
-   (C) 2000-2007 Hans Lub
+   (C) 2000-2025 Hans Lub
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -54,7 +54,7 @@ mystrlcpy(char *dst, const char *src, size_t size)
 void
 mystrlcat(char *dst, const char *src, size_t size)
 {
-#ifdef HAVE_STRLCAT
+#if HAVE_DECL_STRLCAT
   strlcat(dst, src, size);
   dst[size - 1] = '\0';         /* we don't check for truncation, just assure '\0'-termination. */
 #else
@@ -107,6 +107,7 @@ strifnull(char *string)
    successively copy the three arguments into it */
 
 char *
+
 add3strings(const char *str1, const char *str2, const char *str3)
 {
   int size;
@@ -155,13 +156,13 @@ mybasename(const char *filename)
   free(filename_copy);
   return result; /* basename on HP-UX is toxic: the result will be overwritten by subsequent invocations! */
 #else
-  char *p;
+  const char *p;
 
   /* find last '/' in name (if any) */
   for (p = filename + strlen(filename) - 1; p > filename; p--)
     if (*(p - 1) == '/')
       break;
-  return p;
+  return mysavestring(p);
 #endif
 }
 
@@ -258,7 +259,6 @@ void test_haystack(const char *haystack, const char* needle) {
 /* split_with("a bee    cee"," ") returns a pointer to an array {"a", "bee",  "cee", NULL} on the heap */
 
 char **
-
 split_with(const char *string, const char *delimiters) {
   const char *s;
   char *token, **pword;
@@ -284,7 +284,8 @@ unsplit_with(int n, char **strings, const char *delim) {
 
 /* split_with("a\t\tbla", '\t') returns {"a" "bla", NULL}, but we want {"a", "", "bla", NULL} for filter completion.
    We write a special version (can be freed with free_splitlist), that optionally checks the number of components (if expected_count > 0) */
-char **split_on_single_char(const char *string, char c, int expected_count) {
+char **
+split_on_single_char(const char *string, char c, int expected_count) {
   /* the 1st +1 for the last element ("bla"), the 2nd +1 for the marker element (NULL) */
   char **list = mymalloc((count_char_occurrences(string,c) + 1 + 1) * sizeof(char **));
   char *stringcopy = mysavestring(string);
@@ -339,16 +340,22 @@ search_and_replace(char *patt, const char *repl, const char *string, int cursorp
                    int *line, int *col)
 {
   int i, j, k;
-  int pattlen = strlen(patt);
-  int replen = strlen(repl);
-  int stringlen = strlen(string);
+  int pattlen, replen, stringlen;
   int cursor_found = FALSE;
   int current_line = 1;
   int current_column = 0;
   size_t scratchsize;
   char *scratchpad, *result;
 
-  assert(patt && repl && string);
+  assert(patt);
+  pattlen = strlen(patt);
+
+  assert(repl);
+  replen = strlen(repl);
+
+  assert(string);
+  stringlen = strlen(string);
+  
   DPRINTF2(DEBUG_READLINE, "string=%s, cursorpos=%d",
            M(string), cursorpos);
   scratchsize = max(stringlen, (stringlen * replen) / pattlen) + 1;     /* worst case : repleng > pattlen and string consists of only <patt> */
@@ -475,7 +482,7 @@ mangle_string_for_debug_log(const char *string, int maxlen)
 
 char *mangle_buffer_for_debug_log(const char *buffer, int length) {
   char *string = mymalloc(length+1);
-  int debug_saved = debug; /* needed by macro MANGLE_LENGTH */
+  int ATTR_ONLY_FOR_DEBUGGING debug_saved = debug; /* needed by macro MANGLE_LENGTH */
   memcpy(string, buffer, length);
   string[length] = '\0';
   return mangle_string_for_debug_log(string, MANGLE_LENGTH);
@@ -768,12 +775,17 @@ copy_next(int n, const char **original, char **copy)
 
 
 /* helper function: returns the number of displayed characters (the "colourless length") of str (which has to have its
-   unprintable sequences marked with RL_PROMPT_*_IGNORE).  Puts a copy without the RL_PROMPT_*_IGNORE characters in
-   *copy_without_ignore_markers (if != NULL)
+   unprintable sequences marked with RL_PROMPT_*_IGNORE).  
+   It works internally by building a list  of visible character cells (struct mbchar_cell, see below) and returning its length.
+   Flattens this list into bytes and points copy_without_ignore_markers  (if  != NULL) to the result
+   if stop_at != 0 then the list building will end  as soon as stop_at cells have been seen, and  point stopptr (if != NULL) 
+   at a copy of str beginning at the first (visible, multibyte) character past the stop position.  
+   
+   This can be used to determine where a (possibly multibyte, coloured) prompt moves to a new line on a narrow terminal.   
 */
 
 int
-colourless_strlen(const char *str, char ** pcopy_without_ignore_markers, int UNUSED(termwidth))
+colourless_strlen(const char *str, char ** pcopy_without_ignore_markers, int UNUSED(termwidth), int stop_at, char **stopptr)
 {
   int visible  = TRUE;
   int length   = strlen(str);
@@ -791,17 +803,22 @@ colourless_strlen(const char *str, char ** pcopy_without_ignore_markers, int UNU
     
   /* The next loop scans str, one multi-byte character at a time, constructing a colourless copy  */
   /* cellptr always points at the next available free cell                                        */
-  for(mbc_initstate(&st), str_ptr = str, colourless_bytes = 0, cellptr = copied_cells;
+  for(mbc_initstate(&st), colourless_length = 0, str_ptr = str, colourless_bytes = 0, cellptr = copied_cells;
       *str_ptr;
       mbc_inc(&str_ptr, &st)) {
     assert (cellptr < copied_cells + length);
+    if (stop_at && stopptr && (colourless_length >= stop_at)) {
+      *stopptr = mysavestring(str_ptr);
+      break;
+    }
+
     switch (*str_ptr) {
     case RL_PROMPT_START_IGNORE:
       visible = FALSE;
-      continue;
+      break;
     case RL_PROMPT_END_IGNORE:
       visible = TRUE;
-      continue;
+      break;
     case '\r':
       if (visible) {                  /* only ever interpret CR (and Backspace) when visible (i.e. outside control sequences) */
         for ( ; cellptr > copied_cells; cellptr--)  
@@ -809,7 +826,7 @@ colourless_strlen(const char *str, char ** pcopy_without_ignore_markers, int UNU
         mbc_initstate(&st);           /* restart with virgin state */
         colourless_bytes = 0;
       }
-      continue;
+      break;
     case '\b':
       if ((visible && cellptr > copied_cells)) {     /* except when invisible, or at beginning of copy ... */
         cellptr -= 1;                                /* ... reset cellptr to previous (multibyte) char     */
@@ -820,22 +837,24 @@ colourless_strlen(const char *str, char ** pcopy_without_ignore_markers, int UNU
         else
           mbc_initstate(&st);                        /* or initial state, if at start of line              */
       }
-      continue;
-    }
-    if (visible) {
-      MBSTATE st_copy   = st;
-      int nbytes        = mbc_charwidth(str_ptr, &st_copy);
-      char *q           = cellptr -> bytes = mymalloc(1 + nbytes);
+      break;
+    default:
+      if (visible) {
+        MBSTATE st_copy   = st;
+        int nbytes        = mbc_charwidth(str_ptr, &st_copy);
+        char *q           = cellptr -> bytes = mymalloc(1 + nbytes);
 
-      colourless_bytes += nbytes;
-      mbc_copy(str_ptr,&q, &st); /* copy the possibly multi-byte character at str_ptr to cellptr -> bytes , incrementing q to just past the copy */
-      *q                = '\0';
-      cellptr -> state  = st; /* remember shift state after reading str_ptr, just in case a backspace comes along later */
-      cellptr           += 1;
+        colourless_bytes += nbytes;
+        mbc_copy(str_ptr,&q, &st); /* copy the possibly multi-byte character at str_ptr to cellptr -> bytes , incrementing q to just past the copy */
+        *q                = '\0';
+        cellptr -> state  = st; /* remember shift state after reading str_ptr, just in case a backspace comes along later */
+        cellptr           += 1;
+      }
+      break;
     }
+    colourless_length = cellptr - copied_cells;
   } /* end of for loop */
 
-  colourless_length = cellptr - copied_cells;
   
   copy_without_ignore_markers = mymalloc(colourless_bytes + 1);
   for (cellptr = copied_cells, copy_ptr = copy_without_ignore_markers, i = 0; i < colourless_length; i++, cellptr++) {
@@ -855,11 +874,23 @@ colourless_strlen(const char *str, char ** pcopy_without_ignore_markers, int UNU
   else
     free(copy_without_ignore_markers);
   
-  
 
   return colourless_length;
 }
 
+
+DEF_UNIT_TEST(test_colourless_strlen) {
+  MAYBE_UNUSED(argv); MAYBE_UNUSED(argc);
+  if (STAGE(TEST_AT_PROGRAM_START))   {
+    char test[] = "\033[0;31mblא\033[0m bla \033[0;33mblא\033[0m";
+    char *result, *copy;
+    int len = colourless_strlen(mark_invisible(test), &copy, 0, 2,  &result);  
+    printf("origineel = '%s', len = %d, copy = '%s', result = '%s'\n", test, len, copy, result);
+    exit(0);
+  }
+}
+
+              
 /* helper function: returns the number of displayed characters (the
    "colourless length") of str (which has its unprintable sequences
    marked with RL_PROMPT_*_IGNORE).
@@ -874,7 +905,7 @@ int
 colourless_strlen_unmarked (const char *str, int termwidth)
 {
   char *marked_str = mark_invisible(str);
-  int colourless_length = colourless_strlen(marked_str, NULL, termwidth);
+  int colourless_length = colourless_strlen(marked_str, NULL, termwidth, 0, NULL);
   free(marked_str);
   return colourless_length;
 }
@@ -912,6 +943,11 @@ get_last_screenline(char *long_line, int termwidth)
   }
 }
 
+
+
+
+
+
 /* lowercase(str) returns lowercased copy of str */
 char *
 lowercase(const char *str) {
@@ -1023,7 +1059,7 @@ decode_field_length(char** ppmessage)
 
 /* Test an invariant: */
 void
-test_field_length_encoding()
+test_field_length_encoding(void)
 {
   int testval = 1423722;
   char *encoded = encode_field_length(testval);
@@ -1357,7 +1393,8 @@ void check_cupcodes(const char *client_output) {
   static int rmcup_len, smcup_len, max_cuplen;
   char *output_copy, *outputptr;
   int cut_here;
-  
+
+
   if (!(commands_children_not_wrapped && term_smcup && term_rmcup))
     return; /* check is impossible or unnecessary */
 
@@ -1369,6 +1406,8 @@ void check_cupcodes(const char *client_output) {
   }     
   outputptr = output_copy = append_and_free_old(still_unchecked, client_output);
 
+  DPRINTF1(DEBUG_READLINE, "check_cupcodes in (%s)", M(outputptr));
+
   /* keep the very end for next time, as it might contain a partial cupcode */
   /* in the worst case we will notice the shortest of the two cupcodes twice */
   cut_here  = max(0, strlen(output_copy) - max_cuplen);
