@@ -33,7 +33,7 @@ static FILE *log_fp;
    back */
 
 void
-yield()
+yield(void)
 {
   DPRINTF0(DEBUG_TERMIO, "yield...");
 #if 0 /* If command writes some output and immediately starts a long computation, sched_yield() will not return for a
@@ -52,7 +52,7 @@ static volatile int signal_handled = FALSE;
 
 #ifdef HAVE_REAL_PSELECT
 
-void zero_select_timeout () {
+void zero_select_timeout (void) {
   signal_handled = TRUE;
 }
 
@@ -60,12 +60,12 @@ void zero_select_timeout () {
 
 
 #define until_hell_freezes  NULL ;
+
 static struct timeval * volatile pmy_select_timeout_tv; /* The SIGCHLD handler sets this variable (see zero_select_timeout() below) to make
-                                                          select() return immediately when a child has died. gcc (4.8 and higher) may optimize it 
-                                                          into a register, which won't work: hence the "volatile" keyword */ 
+                                                          select() return immediately when a child has died. gcc (4.8 and higher) may optimize it                                                           into a register, which won't work: hence the "volatile" keyword */ 
 static struct timeval my_select_timeout_tv;
 
-void zero_select_timeout () {
+void zero_select_timeout (void) {
   my_select_timeout_tv.tv_sec = 0;
   my_select_timeout_tv.tv_usec = 0;
   pmy_select_timeout_tv = &my_select_timeout_tv;
@@ -275,9 +275,9 @@ mysetenv(const char *name, const char *value)
   int return_value = 0;
   
   
-#ifdef HAVE_SETENV
+#if HAVE_DECL_SETENV
   return_value = setenv(name, value, TRUE);   
-#elif defined(HAVE_PUTENV)
+#elif HAVE_DECL_PUTENV
   char *name_is_value = add3strings (name, "=", value);
   return_value = putenv (name_is_value);
 #else /* won't happen, but anyway: */      
@@ -313,7 +313,7 @@ int open_unique_tempfile(const char *suffix, char **tmpfile_name) {
  
   *tmpfile_name = mymalloc(MAXPATHLEN+1);
 
-#ifdef HAVE_MKSTEMPS
+#if HAVE_DECL_MKSTEMPS
   snprintf4(*tmpfile_name, MAXPATHLEN, "%s/%s_%s_XXXXXX%s", tmpdir, program_name, command_name, suffix); 
   tmpfile_fd = mkstemps(*tmpfile_name, strlen(suffix));  /* this will write into *tmpfile_name */
 #else
@@ -456,14 +456,14 @@ filesize(const char *filename)
 
 
 void
-close_logfile()
+close_logfile(void)
 {
   if (log_fp)
     fclose(log_fp);
 }
 
 void
-close_open_files_without_writing_buffers() /* called from child just before exec(command) */
+close_open_files_without_writing_buffers(void) /* called from child just before exec(command) */
 {
   if(log_fp)
     close(fileno(log_fp));  /* don't flush buffers to avoid avoid double double output output */
@@ -480,6 +480,7 @@ timestamp(char *buf, int size)
   static int never_called = 1;
   long diff_usec;
   float diff_sec;
+  MAYBE_UNUSED(size); /* on systems without snprintf */
   
   gettimeofday(&now, NULL);
   if (never_called) {
@@ -534,17 +535,17 @@ get_new_slave_cwd(char **cwd)
 {
   char *possibly_new_cwd = NULL;
   int return_value = 0;
-
   
 #if defined(HAVE_PROC_PID_CWD) /* Linux, Solaris, and FreeBSD with the proc filesystem */
 
   static char *proc_pid_cwd = NULL;
   char readlink_buffer[MAXPATHLEN+1];
+  MAYBE_UNUSED(readlink_buffer);
     
   if (!proc_pid_cwd)
     proc_pid_cwd = add3strings(PROC_MOUNTPOINT, "/", add2strings(as_string(command_pid), "/cwd"));
 
-# ifdef HAVE_READLINK
+# if HAVE_DECL_READLINK
   if  (readlink(proc_pid_cwd, readlink_buffer, MAXPATHLEN) > 0)
     possibly_new_cwd = mysavestring(readlink_buffer);
 # else
@@ -599,7 +600,7 @@ get_new_slave_cwd(char **cwd)
 
 /* change_working_directory() tries to change rlwrap's working directory to the rlwrapped command's current working directory   */
 void
-change_working_directory()
+change_working_directory(void)
 {
   static char *slaves_working_directory = NULL;
   if (!slaves_working_directory)
@@ -644,7 +645,7 @@ void log_fd_info(int fd) {
 
 /* some last-minute checks before we can start */
 void
-last_minute_checks()
+last_minute_checks(void)
 {
   /* flag unhealthy option combinations */
   if (multiline_separator && filter_command)
@@ -701,7 +702,7 @@ static void print_option(char shortopt, char *longopt, char*argument, int option
   /* don't free allocated strings: we'll exit() soon */
 }
 
-static void
+static void ATTR_ONLY_FOR_DEBUGGING
 print_debug_flag(int flag, char *explanation) {
   fprintf(stderr, "    %4d    %s\n", flag, explanation);
 }
@@ -745,6 +746,7 @@ usage(int status)
   print_option('v', "version", NULL, FALSE, NULL);
   print_option('w', "wait-before-prompt", "N", FALSE, "(msec, <0  : patient mode)");
   print_option('W', "polling", NULL, FALSE, NULL);
+  print_option('X', "skip-setctty", NULL, FALSE, NULL);
   print_option('z', "filter", "filter command", FALSE, "('rlwrap -z listing' writes a list of installed filters)");  
   
  
@@ -827,9 +829,9 @@ free_multiple(void *ptr, ...)
 
 
 
-void mysetsid() {
+void mysetsid(void) {
 # ifdef HAVE_SETSID /* c'mon, this is POSIX! */
-  pid_t ret = setsid();
+  pid_t ATTR_ONLY_FOR_DEBUGGING ret = setsid();
 
   DPRINTF2(DEBUG_TERMIO, "setsid() returned %d %s", (int)ret,
            ERRMSG(ret < 0));
